'use strict';

var _require = require('./http'),
    httpGetFn = _require.httpGetFn,
    httpDeleteFn = _require.httpDeleteFn,
    httpPostFn = _require.httpPostFn,
    httpPutFn = _require.httpPutFn;

var _require2 = require('assert'),
    deepEqual = _require2.deepEqual,
    equal = _require2.equal;

var _require3 = require('sinon'),
    stub = _require3.stub;

var cmds = require('../cmds');

var createResponsePromise = function createResponsePromise(payload, init) {
  return Promise.resolve(new Response(JSON.stringify(payload), init));
};

describe('handlers', function () {
  describe('httpGetFn', function () {
    it('should make a get request', function () {
      var get = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));
      var cmd = cmds.httpGet('http://www.example.com', { test: 'header' }, { credentials: 'test' });

      return httpGetFn(get, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header'
          },
          method: 'GET'
        };
        deepEqual(get.firstCall.args[0], 'http://www.example.com');
        deepEqual(get.firstCall.args[1], options);
        deepEqual(result, { foo: 'bar' });
      });
    });

    it('should make a get request and return meta', function () {
      var get = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));
      var cmd = cmds.httpGet('http://www.example.com', { test: 'header' }, { credentials: 'test', meta: true });

      return httpGetFn(get, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header'
          },
          meta: true,
          method: 'GET'
        };
        deepEqual(get.firstCall.args[0], 'http://www.example.com');
        deepEqual(get.firstCall.args[1], options);
        deepEqual(result.payload, { foo: 'bar' });
        deepEqual(result.meta, { status: 200, statusText: 'OK', headers: {} });
      });
    });

    it('should make a get request and throw on non 200 status codes', function () {
      var get = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 500 }));

      var cmd = cmds.httpGet('http://www.example.com', { test: 'header' }, { credentials: 'test' });

      return httpGetFn(get, cmd).catch(function (e) {
        var payload = {
          foo: 'bar'
        };

        var meta = {
          status: 500,
          statusText: 'Internal Server Error',
          headers: {}
        };
        equal(e instanceof Error, true);
        deepEqual(e.payload, payload);
        deepEqual(e.meta, meta);
      });
    });
  });

  describe('httpDeleteFn', function () {
    it('should make a delete request', function () {
      var remove = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpDelete('http://www.example.com', { test: 'header' }, { credentials: 'test' });

      return httpDeleteFn(remove, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header'
          },
          method: 'DELETE'
        };
        deepEqual(remove.firstCall.args[0], 'http://www.example.com');
        deepEqual(remove.firstCall.args[1], options);
        deepEqual(result, { foo: 'bar' });
      });
    });

    it('should make a delete request and return meta', function () {
      var remove = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpDelete('http://www.example.com', { test: 'header' }, { credentials: 'test', meta: true });

      return httpDeleteFn(remove, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header'
          },
          meta: true,
          method: 'DELETE'
        };
        deepEqual(remove.firstCall.args[0], 'http://www.example.com');
        deepEqual(remove.firstCall.args[1], options);
        deepEqual(result.payload, { foo: 'bar' });
        deepEqual(result.meta, { status: 200, statusText: 'OK', headers: {} });
      });
    });

    it('should make a delete request and throw on non 200 status codes', function () {
      var remove = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 500 }));

      var cmd = cmds.httpDelete('http://www.example.com', { test: 'header' }, { credentials: 'test' });

      return httpDeleteFn(remove, cmd).catch(function (e) {
        var payload = {
          foo: 'bar'
        };
        var meta = {
          status: 500,
          statusText: 'Internal Server Error',
          headers: {}
        };

        equal(e instanceof Error, true);
        deepEqual(e.payload, payload);
        deepEqual(e.meta, meta);
      });
    });
  });

  describe('httpPostFn', function () {
    it('should make a post request', function () {
      var post = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpPost('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test' });

      return httpPostFn(post, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header',
            'Content-Type': 'application/json;charset=UTF-8'
          },
          method: 'POST',
          body: JSON.stringify({
            pay: 'load'
          })
        };

        deepEqual(post.firstCall.args[0], 'http://www.example.com');
        deepEqual(post.firstCall.args[1], options);
        deepEqual(result, { foo: 'bar' });
      });
    });

    it('should make a post request and return meta', function () {
      var post = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpPost('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test', meta: true });

      return httpPostFn(post, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            test: 'header',
            'Content-Type': 'application/json;charset=UTF-8'
          },
          meta: true,
          method: 'POST',
          body: JSON.stringify({
            pay: 'load'
          })
        };

        deepEqual(post.firstCall.args[0], 'http://www.example.com');
        deepEqual(post.firstCall.args[1], options);
        deepEqual(result.payload, { foo: 'bar' });
        deepEqual(result.meta, { status: 200, statusText: 'OK', headers: {} });
      });
    });

    it('should make a post request and throw on non 200 status codes', function () {
      var post = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 500 }));

      var cmd = cmds.httpPost('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test' });

      return httpPostFn(post, cmd).catch(function (e) {
        var payload = {
          foo: 'bar'
        };

        var meta = {
          status: 500,
          statusText: 'Internal Server Error',
          headers: {}
        };

        equal(e instanceof Error, true);
        deepEqual(e.payload, payload);
        deepEqual(e.meta, meta);
      });
    });
  });

  describe('httpPutFn', function () {
    it('should make a put request', function () {
      var put = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpPut('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test' });

      return httpPutFn(put, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            'Content-Type': 'application/json;charset=UTF-8',
            test: 'header'
          },
          method: 'PUT',
          body: JSON.stringify({
            pay: 'load'
          })
        };
        deepEqual(put.firstCall.args[0], 'http://www.example.com');
        deepEqual(put.firstCall.args[1], options);
        deepEqual(result, { foo: 'bar' });
      });
    });

    it('should make a put request and return meta', function () {
      var put = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 200 }));

      var cmd = cmds.httpPut('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test', meta: true });

      return httpPutFn(put, cmd).then(function (result) {
        var options = {
          credentials: 'test',
          headers: {
            'Content-Type': 'application/json;charset=UTF-8',
            test: 'header'
          },
          method: 'PUT',
          body: JSON.stringify({
            pay: 'load'
          }),
          meta: true
        };
        deepEqual(put.firstCall.args[0], 'http://www.example.com');
        deepEqual(put.firstCall.args[1], options);
        deepEqual(result.payload, { foo: 'bar' });
        deepEqual(result.meta, { status: 200, statusText: 'OK', headers: {} });
      });
    });

    it('should make a post request and throw on non 200 status codes', function () {
      var put = stub().returns(createResponsePromise({ foo: 'bar' }, { status: 500 }));

      var cmd = cmds.httpPut('http://www.example.com', { pay: 'load' }, { test: 'header' }, { credentials: 'test', meta: true });

      return httpPutFn(put, cmd).catch(function (e) {
        var payload = {
          foo: 'bar'
        };

        var meta = {
          status: 500,
          statusText: 'Internal Server Error',
          headers: {}
        };

        equal(e instanceof Error, true);
        deepEqual(e.payload, payload);
        deepEqual(e.meta, meta);
      });
    });
  });
});
//# sourceMappingURL=http.spec.js.map